//             TECHNOLOGICAL ARTS
//              +1(416) 963-8996 
//	    www.technologicalarts.ca

// This is a demo program used to interface Adapt9S12D and Demo board.

//****************************************************************************
// This program will demonstrate how different notes of keyboard are generated
// on the buzzer of demo board. This program will play a music of Mary Had A
// Little Lamb. As we used fixed pause between each note, it is only close 
// to the actual music. If you wish, you can improve the music by
// editing the delays.
//****************************************************************************

// In this program, modulus down counter is used to generate the keyboard note
// frequencies. An output compare timer is used to control the timming of each
// note's on and off time,

#include <hidef.h>      /* common defines and macros */
#include <mc9s12dp512.h>     /* derivative information */
#pragma LINK_INFO DERIVATIVE "mc9s12dp512"

#include <OC_Timer.h>
#include <M_Timer.h>
#include <NOTES.h>
#include <PORT.h>
#include <PLL.h>

#define PERIOD 24000
#define NOTE_ON_TIME	  250		//350 msec
#define NOTE_OFF_TIME	  255		//360-350= 10 msec off time
                                        
int Count,Note_P,squarewave,q,pause;;

// notes for MARY HAD A LITTLE LAMB.
int NOTE[]={51,49,47,49,51,51,51,49,49,49,51,42,42,42,49,47,49,51,51,51,51,49,49,51,49,47};
const Note_Length=26;

void main(void) 
{ 
 
  PORT_Init();			// initialize PORTs 
  PLL_Initialization();   	// initialize PLL  
  output_compare_initialize();  // initialize Output capture timer
  ModulusDown_Initialization(); // initialize Modulus down Counter

  EnableInterrupts;		// enable interrupt
  while(1)
  	{    
		// wait forever
  	}
}


//&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&
// This is interrupt subroutine for OUTPUTCOMPARE
//&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&
#pragma CODE_SEG __NEAR_SEG NON_BANKED 
 interrupt void toc0_isr(void)
{

TFLG1=0x01;
TC0=TCNT+PERIOD;

	Count++;

 if (Count<=NOTE_ON_TIME)
 {
   pause=1;			      // play note for NOTE_ON_TIME time
 }
 
 if ((Count>NOTE_ON_TIME)&&(Count<=NOTE_OFF_TIME))
 {
   pause=0;			      // discountinue note for 
 }				      // NOTE_OFF_TIME
 if (Count>NOTE_OFF_TIME) 
 {
  Count=0;			      // after NOTE_OFF_TIME play next 
  Note_P=1;			      // NOTE
 }
 
}

#pragma CODE_SEG DEFAULT 


 //&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&
// This is interrupt subroutine for Real Time Interrupt
//&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&
#pragma CODE_SEG __NEAR_SEG NON_BANKED
  interrupt void ModulusDown_ISR(void)
  {
	MCFLG = 0x80;         // clear flag 

	if (pause==1) 	      // if pause is 1 
	{
	  if (squarewave==1)  // generate the square wave of
	    { 		      // note frequency
	      squarewave=0;
	      PTP&=0x7F;
	    } 
	  else	  
	    { 
	      squarewave=1;
	      PTP|=0x80;
	    }
	} 
	
	else 
	{
	 PTP&=0x7F;	      // else stay off for pause
	}
	    
	
	 if (Note_P==1)       			// if NOTE period is over
	  {
	    q++;
	    MCCNT=Select_Note_Freq(NOTE[q]);    // play the next NOTE
	    Note_P=0;
	  }
	  
	 if (q==Note_Length)			// If all NOTES played
	    q=0;				// then repeat the sequence
		 
  }

  #pragma CODE_SEG DEFAULT
  
  
  